/*
* MIT License
* 
* Copyright (c) 2022 nap.software
* 
* Permission is hereby granted, free of charge, to any person obtaining a copy
* of this software and associated documentation files (the "Software"), to deal
* in the Software without restriction, including without limitation the rights
* to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
* copies of the Software, and to permit persons to whom the Software is
* furnished to do so, subject to the following conditions:
* 
* The above copyright notice and this permission notice shall be included in all
* copies or substantial portions of the Software.
* 
* THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
* IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
* FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
* AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
* LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
* OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
* SOFTWARE.
*/
#if !defined(NAPC_RAND_SOURCE_h)
	#define NAPC_RAND_SOURCE_h

	#include <libnapc.h>

	/*!
	 * @name napc_getRandomBitSources
	 * @brief Get source ids for randomness.
	 * @version 1.0.0
	 * @description
	 * Returns a list of all available sources of randomness.
	 * The end of the list is indicated with `-1` (which is not a valid source identifier).
	 * @return Returns a list of integers that can be passed as a second parameter to `napc_getRandomBit`.
	 * @notes
	 * Since this function is hardware dependent, its output also is dependent on the target hardware.
	 * @changelog 1.0.0 17.02.2022 initial version
	 * @example
	 * const int *sources = napc_getRandomBitSources();
	 * 
	 * for (int i = 0; sources[i] >= 0; ++i) {
	 *     napc_printf("Source identifier %d\n", sources[i]);
	 * }
	 */
	const int *napc_getRandomBitSources(void) NAPC_FN_WARNUNUSED_RET();

	/*!
	 * @name napc_getRandomBit
	 * @brief Create a random bit value.
	 * @version 1.0.0
	 * @description
	 * Creates a random bit value (true or false).
	 * @param out Location to put random bit into.
	 * @param source Source to get the randomness from. See `napc_getRandomBitSources`.
	 * @return Returns `true` if `out` contains a new randomly generated bit.
	 * @warning
	 * It's not advised to call this function directly, use `napc_getRandomByte` for generating a random byte instead.
	 * @warning
	 * `out` remains UNMODIFIED if function returned `false`.
	 * @changelog 1.0.0 17.02.2022 initial version
	 * @example
	 * napc_u8 out;
	 * 
	 * if (napc_getRandomBit(&out)) {
	 *     // do something with `out`
	 * }
	 */
	bool napc_getRandomBit(bool *out, int source) NAPC_FN_WARNUNUSED_RET();

	/*!
	 * @name napc_getRandomByte
	 * @brief Create a random byte.
	 * @version 1.0.0
	 * @description
	 * Create a random byte.
	 * @param out Location to put random byte into.
	 * @return Returns `true` if `out` contains a new randomly generated byte.
	 * @warning
	 * `out` remains UNMODIFIED if function returned `false`.
	 * @changelog 1.0.0 17.02.2022 initial version
	 * @example
	 * napc_u8 out;
	 * 
	 * if (napc_getRandomByte(&out)) {
	 *     // do something with `out`
	 * }
	 */
	bool napc_getRandomByte(napc_u8 *out) NAPC_FN_WARNUNUSED_RET();
#endif
